@@ -41,10 +41,10 @@
 
 #include <iostream>
 
-class ImageStreamStateCallback : public osg::Drawable::UpdateCallback {
+class ImageStreamStateCallback : public osg::DrawableUpdateCallback {
 public:
     ImageStreamStateCallback(osgText::Text* text, osg::ImageStream* is)
-        : osg::Drawable::UpdateCallback()
+        : osg::DrawableUpdateCallback()
         , _text(text)
         , _imageStream(is)
         , _fps(0)
@@ -129,7 +129,7 @@ protected:
 
         osg::Vec4Array* c = dynamic_cast<osg::Vec4Array*>(geo->getColorArray());
         if (c) (*c)[0] = color;
-        geo->dirtyDisplayList();
+        geo->dirtyGLObjects();
         c->dirty();
     }
 
@@ -141,7 +141,7 @@ protected:
 };
 
 
-bool MovieEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* nv)
+bool MovieEventHandler::handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* /*nv*/)
 {
     switch(ea.getEventType())
     {
@@ -350,7 +350,7 @@ public:
                 (*tex_coords)[2].set(r,b);
                 (*tex_coords)[3].set(r,t);
                 tex_coords->dirty();
-                _geo->dirtyDisplayList();
+                _geo->dirtyGLObjects();
             }
         }
     }
@@ -490,7 +490,7 @@ public:
         loadSlide(_currentFile);
     }
 
-    bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& aa, osg::Object*, osg::NodeVisitor* nv)
+    bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& /*aa*/, osg::Object*, osg::NodeVisitor* /*nv*/)
     {
         switch(ea.getEventType())
         {
