@@ -50,6 +50,7 @@ int main( int argc, char **argv )
     // left window + left slave camera
     {
         osg::ref_ptr<osg::GraphicsContext::Traits> traits = new osg::GraphicsContext::Traits;
+
         traits->x = xoffset + 0;
         traits->y = yoffset + 0;
         traits->width = 600;
@@ -57,6 +58,8 @@ int main( int argc, char **argv )
         traits->windowDecoration = true;
         traits->doubleBuffer = true;
         traits->sharedContext = 0;
+        traits->readDISPLAY();
+        traits->setUndefinedScreenDetailsToDefaultScreen();
 
         osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext(traits.get());
 
@@ -81,6 +84,8 @@ int main( int argc, char **argv )
         traits->windowDecoration = true;
         traits->doubleBuffer = true;
         traits->sharedContext = 0;
+        traits->readDISPLAY();
+        traits->setUndefinedScreenDetailsToDefaultScreen();
 
         osg::ref_ptr<osg::GraphicsContext> gc = osg::GraphicsContext::createGraphicsContext(traits.get());
 
