@@ -1,13 +1,13 @@
 /* -*-c++-*- OpenThreads library, Copyright (C) 2002 - 2007  The Open Thread Group
  *
- * This library is open source and may be redistributed and/or modified under  
- * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
+ * This library is open source and may be redistributed and/or modified under
+ * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
  * (at your option) any later version.  The full license is in LICENSE file
  * included with this distribution, and on the openscenegraph.org website.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * OpenSceneGraph Public License for more details.
 */
 
@@ -31,8 +31,13 @@ class PThreadBarrierPrivateData {
 
 private:
 
-    PThreadBarrierPrivateData() {};
-    
+    PThreadBarrierPrivateData(int numThreads)
+    {
+        cnt = 0;
+        phase = 0;
+        maxcnt = numThreads;
+    };
+
     virtual ~PThreadBarrierPrivateData() {};
 
     pthread_cond_t     cond;            // cv for waiters at barrier
