@@ -34,7 +34,10 @@ typedef char TCHAR;
     #include <direct.h> // for _mkdir
 
     #define mkdir(x,y) _mkdir((x))
+
+#if !defined(__MINGW32__)
     #define stat64 _stati64
+#endif
 
     // set up for windows so acts just like unix access().
 #ifndef F_OK
@@ -92,6 +95,10 @@ typedef char TCHAR;
     #endif
 #endif
 
+#if defined(__ANDROID__)
+    #define stat64 stat
+#endif
+
     // set up _S_ISDIR()
 #if !defined(S_ISDIR)
 #  if defined( _S_IFDIR) && !defined( __S_IFDIR)
@@ -168,13 +175,11 @@ bool osgDB::makeDirectory( const std::string &path )
         }
     }
 
-    std::string dir = path;
     std::stack<std::string> paths;
-    while( true )
+    for(std::string dir = path;
+        !dir.empty();
+        dir = getFilePath(dir))
     {
-        if( dir.empty() )
-            break;
-
 #ifdef OSG_USE_UTF8_FILENAME
         if( _wstat64( OSGDB_STRING_TO_FILENAME(dir).c_str(), &stbuf ) < 0 )
 #else
@@ -193,7 +198,6 @@ bool osgDB::makeDirectory( const std::string &path )
                     return false;
             }
         }
-        dir = getFilePath(std::string(dir));
     }
 
     while( !paths.empty() )
@@ -338,6 +342,11 @@ std::string osgDB::findFileInPath(const std::string& filename, const FilePathLis
         OSG_DEBUG << "itr='" <<*itr<< "'\n";
         std::string path = itr->empty() ? filename : concatPaths(*itr, filename);
 
+#ifdef WIN32
+        // if combined file path exceeds MAX_PATH then ignore as it's not a legal path otherwise subsequent IO calls with this path may result in undefined behavior
+        if (path.length()>MAX_PATH) continue;
+#endif
+
         path = getRealPath(path);
 
         OSG_DEBUG << "FindFileInPath() : trying " << path << " ...\n";
@@ -418,7 +427,19 @@ std::string osgDB::findFileInDirectory(const std::string& fileName,const std::st
         realFileName = getSimpleFileName(fileName);
     }
 
-    OSG_DEBUG << "findFileInDirectory() : looking for " << realFileName << " in " << realDirName << "...\n";
+
+    if (realDirName.size()>2)
+    {
+        char lastCharacter = realDirName[realDirName.size()-1];
+        bool trimLastCharacter = lastCharacter=='/' ||  lastCharacter=='\\';
+        if (trimLastCharacter)
+        {
+            realDirName.erase(realDirName.size()-1, 1);
+            OSG_DEBUG << "findFileInDirectory() Trimming last character of filepath, now realDirName="<<realDirName<<std::endl;
+        }
+    }
+
+    OSG_DEBUG << "findFileInDirectory() : looking for " << realFileName << " in " << realDirName << std::endl;
 
     if (realDirName.empty())
     {
@@ -601,26 +622,26 @@ osgDB::DirectoryContents osgDB::expandWildcardsInFilename(const std::string& fil
     return contents;
 }
 
-osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std::string & destination)
+osgDB::CopyFileResult osgDB::copyFile(const std::string & source, const std::string & destination)
 {
     if (source.empty() || destination.empty())
     {
         OSG_INFO << "copyFile(): Empty file name." << std::endl;
-        return FileOpResult::BAD_ARGUMENT;
+        return COPY_FILE_BAD_ARGUMENT;
     }
 
     // Check if source and destination are the same
     if (source == destination || osgDB::getRealPath(source) == osgDB::getRealPath(destination))
     {
         OSG_INFO << "copyFile(): Source and destination point to the same file: source=" << source << ", destination=" << destination << std::endl;
-        return FileOpResult::SOURCE_EQUALS_DESTINATION;
+        return COPY_FILE_SOURCE_EQUALS_DESTINATION;
     }
 
     // Check if source file exists
     if (!osgDB::fileExists(source))
     {
         OSG_INFO << "copyFile(): Source file does not exist: " << source << std::endl;
-        return FileOpResult::SOURCE_MISSING;
+        return COPY_FILE_SOURCE_MISSING;
     }
 
     // Open source file
@@ -628,7 +649,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     if (!fin)
     {
         OSG_NOTICE << "copyFile(): Can't read source file: " << source << std::endl;
-        return FileOpResult::SOURCE_NOT_OPENED;        // Return success since it's not an output error.
+        return COPY_FILE_SOURCE_NOT_OPENED;        // Return success since it's not an output error.
     }
 
     // Ensure the directory exists or else the FBX SDK will fail
@@ -642,7 +663,7 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     if (!fout)
     {
         OSG_NOTICE << "copyFile(): Can't write destination file: " << destination << std::endl;
-        return FileOpResult::DESTINATION_NOT_OPENED;
+        return COPY_FILE_DESTINATION_NOT_OPENED;
     }
 
     // Copy file
@@ -657,16 +678,16 @@ osgDB::FileOpResult::Value osgDB::copyFile(const std::string & source, const std
     if (!fout.good())
     {
         OSG_NOTICE << "copyFile(): Error writing destination file: " << destination << std::endl;
-        return FileOpResult::WRITE_ERROR;
+        return COPY_FILE_WRITE_ERROR;
     }
 
     if (!fin.eof())
     {
         OSG_NOTICE << "copyFile(): Error reading source file: " << source << std::endl;
-        return FileOpResult::READ_ERROR;
+        return COPY_FILE_READ_ERROR;
     }
 
-    return FileOpResult::OK;
+    return COPY_FILE_OK;
 }
 
 
@@ -1016,7 +1037,7 @@ bool osgDB::containsCurrentWorkingDirectoryReference(const FilePathList& paths)
         }
 
         appendInstallationLibraryFilePaths(filepath);
-        
+
         // Since this is currently the only Objective-C code in the
         // library, we need an autoreleasepool for obj-c memory management.
         // If more Obj-C is added, we might move this pool to another
@@ -1036,7 +1057,7 @@ bool osgDB::containsCurrentWorkingDirectoryReference(const FilePathList& paths)
         // the Obj-C strings must be converted down to C strings so
         // C++ can make them into C++ strings.
         filepath.push_back( [myBundlePlugInPath UTF8String] );
-        
+
         // add all application-support-folders
         NSArray *systemSearchPaths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSAllDomainsMask, YES);
 
@@ -1117,7 +1138,7 @@ bool osgDB::containsCurrentWorkingDirectoryReference(const FilePathList& paths)
 
         // Next, check the User's Application Support folder
         int domains[3] = { kUserDomain, kLocalDomain, kNetworkDomain };
-        
+
         for(unsigned int domain_ndx = 0; domain_ndx < 3; ++domain_ndx)
         {
             errCode = FSFindFolder( domains[domain_ndx], kApplicationSupportFolderType, kDontCreateFolder, &f );
