@@ -66,18 +66,18 @@ void addDrawableAndReverseWindingOrder( osg::Geode* geode )
     // Replace double sided polygons by duplicating the drawables and inverting the normals.
     std::vector<osg::Geometry*> new_drawables;
 
-    for (size_t i=0; i<geode->getNumDrawables(); ++i)
+    for (size_t di=0; di<geode->getNumDrawables(); ++di)
     {
-        const osg::Geometry* geometry = dynamic_cast<const osg::Geometry*>(geode->getDrawable(i));
+        const osg::Geometry* geometry = dynamic_cast<const osg::Geometry*>(geode->getDrawable(di));
         if(geometry)
         {
             osg::Geometry* geom = new osg::Geometry(*geometry
                 , osg::CopyOp::DEEP_COPY_ARRAYS | osg::CopyOp::DEEP_COPY_PRIMITIVES);
             new_drawables.push_back(geom);
 
-            for( size_t i = 0; i < geom->getNumPrimitiveSets( ); ++i )
+            for( size_t pi = 0; pi < geom->getNumPrimitiveSets( ); ++pi )
             {
-                osg::DrawArrays* drawarray = dynamic_cast<osg::DrawArrays*>( geom->getPrimitiveSet( i ) );
+                osg::DrawArrays* drawarray = dynamic_cast<osg::DrawArrays*>( geom->getPrimitiveSet( pi ) );
                 if( drawarray )
                 {
                     GLint first = drawarray->getFirst();
@@ -474,6 +474,24 @@ protected:
         {
           _geometry->setUserValue("<UA:FID>", feature);
         }
+ 
+        // terrain 
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isTerrain())
+        {
+          _geometry->setUserValue("<UA:Terrain>", true);
+        }
+
+        // roofline 
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isRoofline())
+        {
+          _geometry->setUserValue("<UA:Roofline>", true);
+        }
+    
+        // footprint 
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isFootprint())
+        {
+          _geometry->setUserValue("<UA:Footprint>", true);
+        }
 
         // Shaders
         if (shaderIndex >= 0)
@@ -957,7 +975,7 @@ protected:
         int materialIndex = in.readInt16(-1);
         int16 surface = in.readInt16();
         int16 feature = in.readInt16();
-        /*int32 IRMaterial =*/ in.readInt32(-1);
+        int32 IRMaterial = in.readInt32(-1);
         _transparency = in.readUInt16(0);
         // version > 13
         /*uint8 influenceLOD =*/ in.readUInt8();
@@ -1048,6 +1066,12 @@ protected:
           _geode->setUserValue("<UA:IRC>", IRColor);
         }
 
+        // IR Material ID (IRM)
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != IRMaterial)
+        {
+          _geode->setUserValue("<UA:IRM>", IRMaterial);
+        }
+
         // surface (SMC)
         if (document.getPreserveNonOsgAttrsAsUserData() && 0 != surface)
         {
@@ -1060,6 +1084,24 @@ protected:
           _geode->setUserValue("<UA:FID>", feature);
         }
 
+        // terrain
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isTerrain())
+        {
+          _geode->setUserValue("<UA:Terrain>", true);
+        }
+
+        // roofline 
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isRoofline())
+        {
+          _geode->setUserValue("<UA:Roofline>", true);
+        }
+
+        // footprint 
+        if (document.getPreserveNonOsgAttrsAsUserData() && 0 != isFootprint())
+        {
+          _geode->setUserValue("<UA:Footprint>", true);
+        }
+
         // Shaders
         if (shaderIndex >= 0)
         {
