@@ -142,6 +142,7 @@ EasyCurl::EasyCurl()
     _previousHttpAuthentication = 0;
     _connectTimeout = 0; // no timeout by default.
     _timeout = 0;
+    _sslVerifyPeer = 1L;
 
     _curl = curl_easy_init();
 
@@ -252,6 +253,9 @@ void EasyCurl::setOptions(const std::string& proxyAddress, const std::string& fi
         curl_easy_setopt(_curl, CURLOPT_PROXY, proxyAddress.c_str()); //Sets proxy address and port on libcurl
     }
 
+    // setting ssl verify peer (default is enabled)
+    curl_easy_setopt(_curl, CURLOPT_SSL_VERIFYPEER, _sslVerifyPeer);
+
     const osgDB::AuthenticationDetails* details = authenticationMap ?
         authenticationMap->getAuthenticationDetails(fileName) :
         0;
@@ -374,22 +378,31 @@ osgDB::ReaderWriter::ReadResult EasyCurl::processResponse(CURLcode res, const st
 
 ReaderWriterCURL::ReaderWriterCURL()
 {
+    // initialize curl to ensure it's done single threaded
+    curl_global_init(CURL_GLOBAL_ALL);
+
     supportsProtocol("http","Read from http port using libcurl.");
     supportsProtocol("https","Read from https port using libcurl.");
     supportsProtocol("ftp","Read from ftp port using libcurl.");
     supportsProtocol("ftps","Read from ftps port using libcurl.");
-    
+
     supportsExtension("curl","Pseudo file extension, used to select curl plugin.");
     supportsExtension("*","Passes all read files to other plugins to handle actual model loading.");
     supportsOption("OSG_CURL_PROXY","Specify the http proxy.");
     supportsOption("OSG_CURL_PROXYPORT","Specify the http proxy port.");
     supportsOption("OSG_CURL_CONNECTTIMEOUT","Specify the connection timeout duration in seconds [default = 0 = not set].");
     supportsOption("OSG_CURL_TIMEOUT","Specify the timeout duration of the whole transfer in seconds [default = 0 = not set].");
+    supportsOption("OSG_CURL_SSL_VERIFYPEER","Specify ssl verification peer [default = 1 = set].");
 }
 
 ReaderWriterCURL::~ReaderWriterCURL()
 {
     //OSG_NOTICE<<"ReaderWriterCURL::~ReaderWriterCURL()"<<std::endl;
+
+    _threadCurlMap.clear();
+
+    // clean up curl
+    curl_global_cleanup();
 }
 
 osgDB::ReaderWriter::WriteResult ReaderWriterCURL::writeFile(const osg::Object& obj, osgDB::ReaderWriter* rw, std::ostream& fout, const osgDB::ReaderWriter::Options *options) const
@@ -419,6 +432,12 @@ osgDB::ReaderWriter::WriteResult ReaderWriterCURL::writeFile(const osg::Object&
     // Serialize obj into an std::stringstream buffer which will be uploaded via HTTP post request.
     std::string fileName = EasyCurl::getFileNameFromURL(fullFileName);
     std::string ext = osgDB::getLowerCaseFileExtension(fileName);
+    if (ext == "curl") 
+    {
+        fileName = osgDB::getNameLessExtension(fileName);
+        ext = osgDB::getLowerCaseFileExtension(fileName);
+    }
+
     osgDB::ReaderWriter* writer = osgDB::Registry::instance()->getReaderWriterForExtension(ext);
     if (!writer) return WriteResult::FILE_NOT_HANDLED;
     osgDB::ReaderWriter::WriteResult result = writeFile(obj, writer, requestBuffer, options);
@@ -428,11 +447,13 @@ osgDB::ReaderWriter::WriteResult ReaderWriterCURL::writeFile(const osg::Object&
     std::string proxyAddress;
     long connectTimeout = 0;
     long timeout = 0;
-    getConnectionOptions(options, proxyAddress, connectTimeout, timeout);
+    long sslVerifyPeer = 1;
+    getConnectionOptions(options, proxyAddress, connectTimeout, timeout, sslVerifyPeer);
     EasyCurl::StreamObject sp(&responseBuffer, &requestBuffer, std::string());
     EasyCurl& easyCurl = getEasyCurl();
     easyCurl.setConnectionTimeout(connectTimeout);
     easyCurl.setTimeout(timeout);
+    easyCurl.setSSLVerifyPeer(sslVerifyPeer);
 
     // Output requestBuffer via curl, and return responseBuffer in message of result.
     return easyCurl.write(proxyAddress, fullFileName, sp, options);
@@ -452,7 +473,11 @@ osgDB::ReaderWriter::ReadResult ReaderWriterCURL::readFile(ObjectType objectType
     return ReadResult::FILE_NOT_HANDLED;
 }
 
-void ReaderWriterCURL::getConnectionOptions(const osgDB::ReaderWriter::Options *options, std::string& proxyAddress, long& connectTimeout, long& timeout) const
+void ReaderWriterCURL::getConnectionOptions(const osgDB::ReaderWriter::Options *options,
+    std::string& proxyAddress,
+    long& connectTimeout,
+    long& timeout,
+    long& sslVerifyPeer) const
 {
     if (options)
     {
@@ -469,8 +494,11 @@ void ReaderWriterCURL::getConnectionOptions(const osgDB::ReaderWriter::Options *
                 connectTimeout = atol(opt.substr( index+1 ).c_str()); // this will return 0 in case of improper format.
             else if( opt.substr( 0, index ) == "OSG_CURL_TIMEOUT" )
                 timeout = atol(opt.substr( index+1 ).c_str()); // this will return 0 in case of improper format.
+            else if( opt.substr(0, index) == "OSG_CURL_SSL_VERIFYPEER" )
+                sslVerifyPeer = atol(opt.substr( index+1 ).c_str()); // this will return 0 in case of improper format.
         }
 
+
         //Setting Proxy by OSG Options
         if(!optProxy.empty())
         {
@@ -529,7 +557,8 @@ osgDB::ReaderWriter::ReadResult ReaderWriterCURL::readFile(ObjectType objectType
     std::string proxyAddress;
     long connectTimeout = 0;
     long timeout = 0;
-    getConnectionOptions(options, proxyAddress, connectTimeout, timeout);
+    long sslVerifyPeer = 1;
+    getConnectionOptions(options, proxyAddress, connectTimeout, timeout, sslVerifyPeer);
 
     bool uncompress = false;
 
@@ -568,6 +597,7 @@ osgDB::ReaderWriter::ReadResult ReaderWriterCURL::readFile(ObjectType objectType
     // setup the timeouts:
     easyCurl.setConnectionTimeout(connectTimeout);
     easyCurl.setTimeout(timeout);
+    easyCurl.setSSLVerifyPeer(sslVerifyPeer);
 
     ReadResult curlResult = easyCurl.read(proxyAddress, fileName, sp, options);
 
@@ -706,8 +736,7 @@ bool ReaderWriterCURL::read(std::istream& fin, std::string& destination) const
 }
 #endif
 
-size_t empty_write_data(const char *buffer, size_t size, size_t nmemb, char
-*userp)
+size_t empty_write_data(const char* /*buffer*/, size_t size, size_t nmemb, char* /*userp*/)
 {
     return size*nmemb;
 }
@@ -726,12 +755,12 @@ bool ReaderWriterCURL::fileExists(const std::string& filename, const osgDB::Opti
         curl_easy_setopt(curl_handle, CURLOPT_WRITEFUNCTION, &empty_write_data);
 
         int result = curl_easy_perform(curl_handle);
-        
+
         long http_return_code(0);
         curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &http_return_code);
-        
+
         curl_easy_cleanup(curl_handle);
-        
+
         return ((result == 0) && ((http_return_code == 200) || (http_return_code == 0)));
     }
     else
