@@ -140,6 +140,9 @@ public:
     virtual ReadResult readImage(const std::string & filename, const osgDB::ReaderWriter::Options* options) const
     {
         const std::string ext = osgDB::getLowerCaseFileExtension(filename);
+        const std::string pro = osgDB::getServerProtocol(filename);
+        if (!acceptsExtension(ext) && !acceptsProtocol(pro)) return ReadResult::FILE_NOT_HANDLED;
+
         if (ext=="ffmpeg") return readImage(osgDB::getNameLessExtension(filename),options);
 
         osg::ref_ptr<osgFFmpeg::FFmpegParameters> parameters(new osgFFmpeg::FFmpegParameters);
