@@ -5,7 +5,7 @@
 #include "ReadWrite.h"
 
 namespace ive{
-class ConvexPlanarPolygon : public osg::ConvexPlanarPolygon, public ReadWrite {
+class ConvexPlanarPolygon : public osg::ConvexPlanarPolygon {
 public:
 	void write(DataOutputStream* out);
 	void read(DataInputStream* in);
