@@ -265,7 +265,7 @@ typedef int trpgMatRef;
 TX_EXDECL class TX_CLDECL trpg2dPoint {
 public:
     double x, y;
-    trpg2dPoint (void) { };
+    trpg2dPoint (void) : x(0.0),y(0.0) { };
     trpg2dPoint (double in_x,double in_y) { x = in_x; y = in_y; };
     bool operator==(const trpg2dPoint& pt ) const {
         if ( x != pt.x ) return false;
@@ -278,14 +278,14 @@ public:
 TX_EXDECL class TX_CLDECL trpg2iPoint {
 public:
     int x,y;
-    trpg2iPoint (void) { };
+    trpg2iPoint (void) : x(0),y(0) { };
     trpg2iPoint (int in_x,int in_y) {x = in_x; y = in_y;};
 };
 /* Double precision 3 dimensional point. */
 TX_EXDECL class TX_CLDECL trpg3dPoint {
 public:
     double x,y,z;
-    trpg3dPoint(void) { };
+    trpg3dPoint(void) : x(0.0),y(0.0),z(0.0) { };
     trpg3dPoint(double in_x,double in_y,double in_z) {x = in_x; y = in_y; z = in_z;}
     bool operator==(const trpg3dPoint& pt ) const {
         if ( x != pt.x ) return false;
@@ -299,7 +299,7 @@ public:
 TX_EXDECL class TX_CLDECL trpgColor {
 public:
     trpgColor(float64 r,float64 g,float64 b) {red = r; green = g; blue = b;}
-    trpgColor(void) { };
+    trpgColor(void) : red(1.0), green(1.0), blue(1.0) { };
     bool operator==(const trpgColor& color) {
         if ( color.red != red )        return false;
         if ( color.green != green ) return false;
@@ -310,7 +310,7 @@ public:
     float64 red,green,blue;
 };
 
-// Used to specify machine endianess
+// Used to specify machine endianness
 typedef enum {LittleEndian,BigEndian} trpgEndian;
 
 /*     This is a base class for an abstract buffer type.
@@ -357,7 +357,7 @@ public:
     virtual void Add(const std::string &);
 
     /* Endianness is something the child class buffer type must set and use.
-        This function returns the endiannes of the current buffer. */
+        This function returns the endianness of the current buffer. */
     virtual trpgEndian GetEndian(void) { return ness; }
 
 protected:
@@ -395,7 +395,7 @@ public:
         */
     virtual void Add(const char *);
     // Same as const char * version
-    virtual void Add(std::string &);
+    virtual void Add(const std::string &);
     // Add a 32 bit float to the buffer
     virtual void Add(float32);
     // Add a 64 bit float to the buffer
@@ -444,7 +444,7 @@ protected:
 };
 
 /* This is a virtual base class for reading data from a device.
-    The device implementation is left as an excercise to the sub class.
+    The device implementation is left as an exercise to the sub class.
     This class contains methods for getting data that must be filled in
     as well as helper methods that call those.
     {group:Low Level I/O}
@@ -518,7 +518,7 @@ protected:
     /* This virtual method must be filled in by the subclass so that SkipToLimit
         will work correctly. */
     virtual bool Skip(int) = 0;
-    /* Utility function that must be called after a successfull read to update
+    /* Utility function that must be called after a successful read to update
         the outside imposed read limits. */
     virtual void UpdateLimits(int);
     std::vector<int> limits;
@@ -562,7 +562,7 @@ TX_EXDECL class TX_CLDECL trpgCheckable {
 public:
     trpgCheckable(void);
     virtual ~trpgCheckable(void);
-    // Returns the state of the valid flag, or can be overriden by a subclass to do a more complex check.
+    // Returns the state of the valid flag, or can be overridden by a subclass to do a more complex check.
     bool isValid(void) const;
 
     virtual TeAttrHdl GetHandle() const {
@@ -606,7 +606,7 @@ public:
     /* The Write method is a virtual that must be filled in by the subclass.
         It takes a trpgWriteBuffer and should return true on success. */
     virtual bool        Write(trpgWriteBuffer &) = 0;
-    /* The Read method should be overriden by a subclass.  It should read
+    /* The Read method should be overridden by a subclass.  It should read
         the contents of the given trpgReadBuffer up to the current limit
         into itself.  It must return true on success. */
     virtual bool        Read(trpgReadBuffer &) { return false;};
@@ -618,11 +618,11 @@ public:
      */
     virtual bool        Print(trpgPrintBuffer &) const { return true; }
 
-    const char *getErrMess() const {if(errMess[0] == '\0') return 0;else return &errMess[0];}
+    const char *getErrMess() const { return errMess.empty() ? 0 : errMess.c_str();}
 
 protected:
 
-    mutable char errMess[512];
+    mutable std::string errMess;
 };
 
 /* Pointer into a trpgwAppFile.  The full name of the file
