@@ -420,7 +420,8 @@ void IncrementalCompileOperation::CompileSet::buildCompileMap(ContextSet& contex
 {
     if (contexts.empty() || !_subgraphToCompile) return;
 
-    StateToCompile stc(mode);
+    StateToCompile stc(mode, _markerObject.get());
+
     _subgraphToCompile->accept(stc);
 
     buildCompileMap(contexts, stc);
@@ -579,6 +580,9 @@ void IncrementalCompileOperation::add(CompileSet* compileSet, bool callBuildComp
 {
     if (!compileSet) return;
 
+    // pass on the markerObject to the CompileSet
+    compileSet->_markerObject = _markerObject;
+
     if (compileSet->_subgraphToCompile.valid())
     {
         // force a compute of the bound of the subgraph to avoid the update traversal from having to do this work
@@ -677,9 +681,6 @@ void IncrementalCompileOperation::operator () (osg::GraphicsContext* context)
     OSG_NOTIFY(level)<<"    currentTime = "<<currentTime<<std::endl;
     OSG_NOTIFY(level)<<"    currentElapsedFrameTime = "<<currentElapsedFrameTime<<std::endl;
 
-    double _flushTimeRatio(0.5);
-    double _conservativeTimeRatio(0.5);
-
     double availableTime = std::max((targetFrameTime - currentElapsedFrameTime)*_conservativeTimeRatio,
                                     minimumTimeAvailableForGLCompileAndDeletePerFrame);
 
