@@ -437,7 +437,7 @@ namespace PlaneIntersectorUtils
             }
         }
 
-        inline void operator () (const osg::Vec3& v1,const osg::Vec3& v2,const osg::Vec3& v3, bool)
+        inline void operator () (const osg::Vec3& v1,const osg::Vec3& v2,const osg::Vec3& v3)
         {
 
             if (_limitOneIntersection && _hit) return;
@@ -468,21 +468,21 @@ namespace PlaneIntersectorUtils
 
             if (numOnPlane==3)
             {
-                // triangle lives wholy in the plane
+                // triangle lives wholly in the plane
                 OSG_NOTICE<<"3"<<std::endl;
                 return;
             }
 
             if (numOnPlane==2)
             {
-                // one edge lives wholy in the plane
+                // one edge lives wholly in the plane
                 OSG_NOTICE<<"2"<<std::endl;
                 return;
             }
 
             if (numOnPlane==1)
             {
-                // one point lives wholy in the plane
+                // one point lives wholly in the plane
                 OSG_NOTICE<<"1"<<std::endl;
                 return;
             }
