@@ -334,7 +334,7 @@ struct TraversalOrderFunctor
 {
     bool operator() (const RenderLeaf* lhs,const RenderLeaf* rhs) const
     {
-        return (lhs->_traversalNumber<rhs->_traversalNumber);
+        return (lhs->_traversalOrderNumber<rhs->_traversalOrderNumber);
     }
 };
 
@@ -555,11 +555,9 @@ bool RenderBin::getStats(Statistics& stats) const
             stats.addMatrix(); // number of matrices
         }
 
-        if (dw)
-        {
-              // then tot up the primitive types and no vertices.
-              dw->accept(stats); // use sub-class to find the stats for each drawable
-        }
+        // then tot up the primitive types and no vertices.
+        dw->accept(stats); // use sub-class to find the stats for each drawable
+
         statsCollected = true;
     }
     stats.addStateGraphs(_stateGraphList.size());
@@ -573,7 +571,7 @@ bool RenderBin::getStats(Statistics& stats) const
             ++dw_itr)
         {
             const RenderLeaf* rl = dw_itr->get();
-            const Drawable* dw= rl->getDrawable();
+            const Drawable* dw = rl->getDrawable();
             stats.addDrawable(); // number of geosets
 
             const Geometry* geom = dw->asGeometry();
@@ -583,11 +581,9 @@ bool RenderBin::getStats(Statistics& stats) const
             }
 
             if (rl->_modelview.get()) stats.addMatrix(); // number of matrices
-            if (dw)
-            {
-                // then tot up the primitive types and no vertices.
-                dw->accept(stats); // use sub-class to find the stats for each drawable
-            }
+
+            // then tot up the primitive types and no vertices.
+            dw->accept(stats); // use sub-class to find the stats for each drawable
         }
         statsCollected = true;
     }
