@@ -19,6 +19,7 @@
 #include <osg/Texture3D>
 #include <osg/TextureRectangle>
 #include <osg/TextureCubeMap>
+#include <osg/ContextData>
 #include <osg/GLExtensions>
 #include <osg/GLU>
 
@@ -247,8 +248,7 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
     int width = static_cast<int>(_viewport->x() + _viewport->width());
     int height = static_cast<int>(_viewport->y() + _viewport->height());
     int depth = 1;
-    osg::Camera::BufferAttachmentMap::iterator itr;
-    for(itr = bufferAttachments.begin();
+    for(osg::Camera::BufferAttachmentMap::iterator itr = bufferAttachments.begin();
         itr != bufferAttachments.end();
         ++itr)
     {
@@ -261,7 +261,7 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
     // OSG_NOTICE<<"RenderStage::runCameraSetUp computed "<<width<<" "<<height<<" "<<depth<<std::endl;
 
     // attach images that need to be copied after the stage is drawn.
-    for(itr = bufferAttachments.begin();
+    for(osg::Camera::BufferAttachmentMap::iterator itr = bufferAttachments.begin();
         itr != bufferAttachments.end();
         ++itr)
     {
@@ -517,12 +517,12 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
             // and before call to glCheckFramebufferStatus
             if ( !colorAttached )
             {
+            #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE)
                 setDrawBuffer( GL_NONE, true );
                 setReadBuffer( GL_NONE, true );
-                #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
-                    glDrawBuffer( GL_NONE );
-                    glReadBuffer( GL_NONE );
-                #endif
+                state.glDrawBuffer( GL_NONE );
+                state.glReadBuffer( GL_NONE );
+            #endif
             }
 
             GLenum status = ext->glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
@@ -537,12 +537,8 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
                 fbo = 0;
 
                 // clean up.
-                double availableTime = 100.0f;
-                double currentTime = state.getFrameStamp()?state.getFrameStamp()->getReferenceTime():0.0;
-                osg::RenderBuffer::flushDeletedRenderBuffers(state.getContextID(),currentTime,availableTime);
-                osg::FrameBufferObject::flushDeletedFrameBufferObjects(state.getContextID(),currentTime,availableTime);
-
-
+                osg::get<osg::GLRenderBufferManager>(state.getContextID())->flushAllDeletedGLObjects();
+                osg::get<osg::GLFrameBufferObjectManager>(state.getContextID())->flushAllDeletedGLObjects();
             }
             else
             {
@@ -555,8 +551,7 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
                 {
                     fbo_multisample->apply(state);
 
-                    GLenum status = ext->glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
-
+                    status = ext->glCheckFramebufferStatus(GL_FRAMEBUFFER_EXT);
                     if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
                     {
                         OSG_NOTICE << "RenderStage::runCameraSetUp(), "
@@ -568,10 +563,8 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
                         _resolveFbo = 0;
 
                         // clean up.
-                        double availableTime = 100.0f;
-                        double currentTime = state.getFrameStamp()?state.getFrameStamp()->getReferenceTime():0.0;
-                        osg::RenderBuffer::flushDeletedRenderBuffers(state.getContextID(),currentTime,availableTime);
-                        osg::FrameBufferObject::flushDeletedFrameBufferObjects(state.getContextID(),currentTime,availableTime);
+                        osg::get<osg::GLRenderBufferManager>(state.getContextID())->flushAllDeletedGLObjects();
+                        osg::get<osg::GLFrameBufferObjectManager>(state.getContextID())->flushAllDeletedGLObjects();
                     }
                     else
                     {
@@ -660,6 +653,7 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
                         traits->depth = 24;
                         depthAttached = true;
                         traits->stencil = 8;
+                        break;
                     }
                     case(osg::Camera::COLOR_BUFFER):
                     {
@@ -764,7 +758,7 @@ void RenderStage::runCameraSetUp(osg::RenderInfo& renderInfo)
 
                 if (pBufferTexture && renderTargetImplementation==osg::Camera::PIXEL_BUFFER_RTT)
                 {
-                   OSG_INFO<<"RenderStage::runCameraSetUp(State&) Assign graphis context to Texture"<<std::endl;
+                   OSG_INFO<<"RenderStage::runCameraSetUp(State&) Assign graphics context to Texture"<<std::endl;
                    pBufferTexture->setReadPBuffer(context.get());
                 }
                 else
@@ -917,13 +911,13 @@ void RenderStage::drawInner(osg::RenderInfo& renderInfo,RenderLeaf*& previous, b
 
     if (!using_multiple_render_targets)
     {
-        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE)
 
             if( getDrawBufferApplyMask() )
-                glDrawBuffer(_drawBuffer);
+                state.glDrawBuffer(_drawBuffer);
 
             if( getReadBufferApplyMask() )
-                glReadBuffer(_readBuffer);
+                state.glReadBuffer(_readBuffer);
 
         #endif
     }
@@ -1003,7 +997,7 @@ void RenderStage::drawInner(osg::RenderInfo& renderInfo,RenderLeaf*& previous, b
                 blitMask, GL_NEAREST);
         }
 
-#if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
+#if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE)
         if (needToBlitColorBuffers)
         {
             for (FrameBufferObject::AttachmentMap::const_iterator
@@ -1013,8 +1007,8 @@ void RenderStage::drawInner(osg::RenderInfo& renderInfo,RenderLeaf*& previous, b
                 osg::Camera::BufferComponent attachment = it->first;
                 if (attachment >=osg::Camera::COLOR_BUFFER0)
                 {
-                    glReadBuffer(GL_COLOR_ATTACHMENT0_EXT + (attachment - osg::Camera::COLOR_BUFFER0));
-                    glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT + (attachment - osg::Camera::COLOR_BUFFER0));
+                    state.glReadBuffer(GL_COLOR_ATTACHMENT0_EXT + (attachment - osg::Camera::COLOR_BUFFER0));
+                    state.glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT + (attachment - osg::Camera::COLOR_BUFFER0));
 
                     ext->glBlitFramebuffer(
                         static_cast<GLint>(_viewport->x()), static_cast<GLint>(_viewport->y()),
@@ -1044,8 +1038,7 @@ void RenderStage::drawInner(osg::RenderInfo& renderInfo,RenderLeaf*& previous, b
         copyTexture(renderInfo);
     }
 
-    std::map< osg::Camera::BufferComponent, Attachment>::const_iterator itr;
-    for(itr = _bufferAttachmentMap.begin();
+    for(std::map< osg::Camera::BufferComponent, Attachment>::const_iterator itr = _bufferAttachmentMap.begin();
         itr != _bufferAttachmentMap.end();
         ++itr)
     {
@@ -1161,7 +1154,7 @@ void RenderStage::draw(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
     if (_camera.valid() && _camera->getInitialDrawCallback())
     {
         // if we have a camera with a initial draw callback invoke it.
-        (*(_camera->getInitialDrawCallback()))(renderInfo);
+        _camera->getInitialDrawCallback()->run(renderInfo);
     }
 
     // note, SceneView does call to drawPreRenderStages explicitly
@@ -1217,7 +1210,7 @@ void RenderStage::draw(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
     if (_camera.valid() && _camera->getPreDrawCallback())
     {
         // if we have a camera with a pre draw callback invoke it.
-        (*(_camera->getPreDrawCallback()))(renderInfo);
+        _camera->getPreDrawCallback()->run(renderInfo);
     }
 
     bool doCopyTexture = _texture.valid() ?
@@ -1278,7 +1271,7 @@ void RenderStage::draw(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
     if (_camera.valid() && _camera->getPostDrawCallback())
     {
         // if we have a camera with a post draw callback invoke it.
-        (*(_camera->getPostDrawCallback()))(renderInfo);
+        _camera->getPostDrawCallback()->run(renderInfo);
     }
 
     if (_graphicsContext.valid() && _graphicsContext != callingContext)
@@ -1316,11 +1309,14 @@ void RenderStage::draw(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
     if (_camera.valid() && _camera->getFinalDrawCallback())
     {
         // if we have a camera with a final callback invoke it.
-        (*(_camera->getFinalDrawCallback()))(renderInfo);
+        _camera->getFinalDrawCallback()->run(renderInfo);
     }
 
     // pop the render stages camera.
     if (_camera.valid()) renderInfo.popCamera();
+
+    // clean up state graph to make sure RenderLeaf etc, can be reused
+    if (_rootStateGraph.valid()) _rootStateGraph->clean();
 }
 
 void RenderStage::drawImplementation(osg::RenderInfo& renderInfo,RenderLeaf*& previous)
@@ -1349,40 +1345,42 @@ void RenderStage::drawImplementation(osg::RenderInfo& renderInfo,RenderLeaf*& pr
     if (_colorMask.valid()) _colorMask->apply(state);
     else glColorMask(GL_TRUE,GL_TRUE,GL_TRUE,GL_TRUE);
 
-    if (_clearMask & GL_COLOR_BUFFER_BIT)
+    if (_clearMask != 0)
     {
-        glClearColor( _clearColor[0], _clearColor[1], _clearColor[2], _clearColor[3]);
-    }
+        if (_clearMask & GL_COLOR_BUFFER_BIT)
+        {
+            glClearColor( _clearColor[0], _clearColor[1], _clearColor[2], _clearColor[3]);
+        }
 
-    if (_clearMask & GL_DEPTH_BUFFER_BIT)
-    {
-        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE)
-            glClearDepth( _clearDepth);
-        #else
-            glClearDepthf( _clearDepth);
+        if (_clearMask & GL_DEPTH_BUFFER_BIT)
+        {
+            #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE)
+                glClearDepth( _clearDepth);
+            #else
+                glClearDepthf( _clearDepth);
+            #endif
+
+            glDepthMask ( GL_TRUE );
+            state.haveAppliedAttribute( osg::StateAttribute::DEPTH );
+        }
+
+        if (_clearMask & GL_STENCIL_BUFFER_BIT)
+        {
+            glClearStencil( _clearStencil);
+            glStencilMask ( ~0u );
+            state.haveAppliedAttribute( osg::StateAttribute::STENCIL );
+        }
+
+        #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE) && !defined(OSG_GL3_AVAILABLE)
+            if (_clearMask & GL_ACCUM_BUFFER_BIT)
+            {
+                glClearAccum( _clearAccum[0], _clearAccum[1], _clearAccum[2], _clearAccum[3]);
+            }
         #endif
 
-        glDepthMask ( GL_TRUE );
-        state.haveAppliedAttribute( osg::StateAttribute::DEPTH );
+        glClear( _clearMask );
     }
 
-    if (_clearMask & GL_STENCIL_BUFFER_BIT)
-    {
-        glClearStencil( _clearStencil);
-        glStencilMask ( ~0u );
-        state.haveAppliedAttribute( osg::StateAttribute::STENCIL );
-    }
-
-    #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GL3_AVAILABLE)
-        if (_clearMask & GL_ACCUM_BUFFER_BIT)
-        {
-            glClearAccum( _clearAccum[0], _clearAccum[1], _clearAccum[2], _clearAccum[3]);
-        }
-    #endif
-
-
-    glClear( _clearMask );
-
     #ifdef USE_SCISSOR_TEST
         glDisable( GL_SCISSOR_TEST );
     #endif
