@@ -24,6 +24,7 @@
 #include <osg/ColorMatrix>
 #include <osg/LightModel>
 #include <osg/CollectOccludersVisitor>
+#include <osg/ContextData>
 
 #include <osg/GLU>
 
@@ -107,6 +108,8 @@ static const GLubyte patternCheckerboard[] = {
 
 SceneView::SceneView(DisplaySettings* ds)
 {
+    _frameStamp = new osg::FrameStamp;
+
     _displaySettings = ds;
 
     _fusionDistanceMode = PROPORTIONAL_TO_SCREEN_DISTANCE;
@@ -145,6 +148,8 @@ SceneView::SceneView(const SceneView& rhs, const osg::CopyOp& copyop):
     osg::Object(rhs,copyop),
     osg::CullSettings(rhs)
 {
+    _frameStamp = new osg::FrameStamp;
+
     _displaySettings = rhs._displaySettings;
 
     _fusionDistanceMode = rhs._fusionDistanceMode;
@@ -308,10 +313,6 @@ void SceneView::init()
 {
     _initCalled = true;
 
-    // force the initialization of the OpenGL extension string
-    // to try and work around a Windows NVidia driver bug circa Oct 2006.
-    osg::isGLExtensionSupported(_renderInfo.getState()->getContextID(),"");
-
     if (_camera.valid() && _initVisitor.valid())
     {
         _initVisitor->reset();
@@ -579,7 +580,7 @@ void SceneView::computeRightEyeViewport(const osg::Viewport *viewport)
 void SceneView::setLightingMode(LightingMode mode)
 {
     if (mode==_lightingMode) return;
-    
+
     osg::StateSet* stateSetToModify = _secondaryStateSet.valid() ? _secondaryStateSet.get() : _globalStateSet.get();
 
     if (_lightingMode!=NO_SCENEVIEW_LIGHT)
@@ -718,13 +719,13 @@ void SceneView::cull()
         else
         {
 
-            if (!_cullVisitorLeft.valid()) _cullVisitorLeft = dynamic_cast<CullVisitor*>(_cullVisitor->clone());
-            if (!_stateGraphLeft.valid()) _stateGraphLeft = dynamic_cast<StateGraph*>(_stateGraph->cloneType());
-            if (!_renderStageLeft.valid()) _renderStageLeft = dynamic_cast<RenderStage*>(_renderStage->clone(osg::CopyOp::DEEP_COPY_ALL));
+            if (!_cullVisitorLeft.valid()) _cullVisitorLeft = _cullVisitor->clone();
+            if (!_stateGraphLeft.valid()) _stateGraphLeft = _stateGraph->cloneStateGraph();
+            if (!_renderStageLeft.valid()) _renderStageLeft = osg::clone(_renderStage.get(), osg::CopyOp::DEEP_COPY_ALL);
 
-            if (!_cullVisitorRight.valid()) _cullVisitorRight = dynamic_cast<CullVisitor*>(_cullVisitor->clone());
-            if (!_stateGraphRight.valid()) _stateGraphRight = dynamic_cast<StateGraph*>(_stateGraph->cloneType());
-            if (!_renderStageRight.valid()) _renderStageRight = dynamic_cast<RenderStage*>(_renderStage->clone(osg::CopyOp::DEEP_COPY_ALL));
+            if (!_cullVisitorRight.valid()) _cullVisitorRight = _cullVisitor->clone();
+            if (!_stateGraphRight.valid()) _stateGraphRight = _stateGraph->cloneStateGraph();
+            if (!_renderStageRight.valid()) _renderStageRight = osg::clone(_renderStage.get(), osg::CopyOp::DEEP_COPY_ALL);
 
             _cullVisitorLeft->setDatabaseRequestHandler(_cullVisitor->getDatabaseRequestHandler());
             _cullVisitorLeft->setClampProjectionMatrixCallback(_cullVisitor->getClampProjectionMatrixCallback());
@@ -738,7 +739,7 @@ void SceneView::cull()
             _cullVisitorRight->setClampProjectionMatrixCallback(_cullVisitor->getClampProjectionMatrixCallback());
             _cullVisitorRight->setTraversalMask(_cullMaskRight);
             computeRightEyeViewport(getViewport());
-            computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitorRight.get(),_stateGraphRight.get(),_renderStageRight.get(),_viewportRight.get());
+            computeNearFar = cullStage(computeRightEyeProjection(getProjectionMatrix()),computeRightEyeView(getViewMatrix()),_cullVisitorRight.get(),_stateGraphRight.get(),_renderStageRight.get(),_viewportRight.get()) | computeNearFar;
 
             if (computeNearFar)
             {
@@ -908,7 +909,7 @@ bool SceneView::cullStage(const osg::Matrixd& projection,const osg::Matrixd& mod
     // note, this would be not required if the rendergraph had been
     // reset at the start of each frame (see top of this method) but
     // a clean has been used instead to try to minimize the amount of
-    // allocation and deleteing of the StateGraph nodes.
+    // allocation and deleting of the StateGraph nodes.
     rendergraph->prune();
 
     // set the number of dynamic objects in the scene.
@@ -926,6 +927,70 @@ void SceneView::releaseAllGLObjects()
     _camera->releaseGLObjects(_renderInfo.getState());
 }
 
+void SceneView::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    struct Resize
+    {
+        unsigned int maxSize;
+
+        Resize(unsigned int ms) : maxSize(ms) {}
+
+        void operator() (osg::Referenced* object)
+        {
+            operator()(dynamic_cast<osg::Object*>(object));
+        }
+        void operator() (osg::Object* object)
+        {
+            if (object) object->resizeGLObjectBuffers(maxSize);
+        }
+    } operation(maxSize);
+
+    operation(_localStateSet.get());
+    operation(_updateVisitor.get());
+    operation(_cullVisitor.get());
+    operation(_stateGraph.get());
+    operation(_renderStage.get());
+    operation(_cullVisitorRight.get());
+    operation(_stateGraphRight.get());
+    operation(_renderStageRight.get());
+    operation(_globalStateSet.get());
+    operation(_secondaryStateSet.get());
+    operation(_cameraWithOwnership.get());
+}
+
+void SceneView::releaseGLObjects(osg::State* state) const
+{
+    if (state && state!=_renderInfo.getState()) return;
+
+    struct Release
+    {
+        osg::State* _state;
+
+        Release(State* state) : _state(state) {}
+
+        void operator() (osg::Referenced* object)
+        {
+            operator()(dynamic_cast<osg::Object*>(object));
+        }
+        void operator() (osg::Object* object)
+        {
+            if (object) object->releaseGLObjects(_state);
+        }
+    } operation(state);
+
+    operation(_localStateSet.get());
+    operation(_updateVisitor.get());
+    operation(_cullVisitor.get());
+    operation(_stateGraph.get());
+    operation(_renderStage.get());
+    operation(_cullVisitorRight.get());
+    operation(_stateGraphRight.get());
+    operation(_renderStageRight.get());
+    operation(_globalStateSet.get());
+    operation(_secondaryStateSet.get());
+    operation(_cameraWithOwnership.get());
+}
+
 void SceneView::flushAllDeletedGLObjects()
 {
     _requiresFlush = false;
@@ -952,7 +1017,7 @@ void SceneView::draw()
 
     osg::State* state = _renderInfo.getState();
 
-    // we in theory should be able to be able to bypass reset, but we'll call it just incase.
+    // we in theory should be able to be able to bypass reset, but we'll call it just in case.
     //_state->reset();
     state->setFrameStamp(_frameStamp.get());
 
@@ -963,11 +1028,7 @@ void SceneView::draw()
 
     state->initializeExtensionProcs();
 
-    osg::Texture::TextureObjectManager* tom = osg::Texture::getTextureObjectManager(state->getContextID()).get();
-    tom->newFrame(state->getFrameStamp());
-
-    osg::GLBufferObjectManager* bom = osg::GLBufferObjectManager::getGLBufferObjectManager(state->getContextID()).get();
-    bom->newFrame(state->getFrameStamp());
+    osg::get<ContextData>(state->getContextID())->newFrame(state->getFrameStamp());
 
     if (!_initCalled) init();
 
@@ -1231,7 +1292,7 @@ void SceneView::draw()
         case(osg::DisplaySettings::HORIZONTAL_INTERLACE):
         case(osg::DisplaySettings::CHECKERBOARD):
             {
-            #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GL3_AVAILABLE)
+            #if !defined(OSG_GLES1_AVAILABLE) && !defined(OSG_GLES2_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE) && !defined(OSG_GLES3_AVAILABLE) && !defined(OSG_GL3_AVAILABLE)
                 if( 0 == ( _camera->getInheritanceMask() & DRAW_BUFFER) )
                 {
                     _renderStageLeft->setDrawBuffer(_camera->getDrawBuffer());
