@@ -16,7 +16,7 @@
 #include <osgUtil/Simplifier>
 
 #include <osgUtil/SmoothingVisitor>
-#include <osgUtil/TriStripVisitor>
+#include <osgUtil/MeshOptimizers>
 
 #include <set>
 #include <list>
@@ -86,9 +86,6 @@ public:
 
     Point* computeInterpolatedPoint(Edge* edge,float r) const
     {
-        Point* point = new Point;
-        float r1 = 1.0f-r;
-        float r2 = r;
         Point* p1 = edge->_p1.get();
         Point* p2 = edge->_p2.get();
 
@@ -98,6 +95,10 @@ public:
             return 0;
         }
 
+        Point* point = new Point;
+        float r1 = 1.0f-r;
+        float r2 = r;
+
         point->_vertex = p1->_vertex * r1 + p2->_vertex * r2;
         unsigned int s = osg::minimum(p1->_attributes.size(),p2->_attributes.size());
         for(unsigned int i=0;i<s;++i)
@@ -266,7 +267,6 @@ public:
     typedef std::vector< osg::ref_ptr<Point> >                                  PointList;
     typedef std::list< osg::ref_ptr<Triangle> >                                 TriangleList;
     typedef std::set< osg::ref_ptr<Triangle> >                                  TriangleSet;
-    typedef std::map< osg::ref_ptr<Triangle>, unsigned int, dereference_less >  TriangleMap;
 
     struct Point : public osg::Referenced
     {
@@ -812,7 +812,6 @@ public:
         osg::ref_ptr<Point> edge_p1 = edge->_p1;
         osg::ref_ptr<Point> edge_p2 = edge->_p2;
 
-        TriangleMap  triangleMap;
         TriangleList triangles_p1;
         TriangleList triangles_p2;
         LocalEdgeList oldEdges;
@@ -1744,14 +1743,17 @@ void Simplifier::simplify(osg::Geometry& geometry, const IndexList& protectedPoi
 {
     OSG_INFO<<"++++++++++++++simplifier************"<<std::endl;
 
+    bool downSample = requiresDownSampling();
+
     EdgeCollapse ec;
-    ec.setComputeErrorMetricUsingLength(getSampleRatio()>=1.0);
+    ec.setComputeErrorMetricUsingLength(!downSample);
     ec.setGeometry(&geometry, protectedPoints);
     ec.updateErrorMetricForAllEdges();
 
     unsigned int numOriginalPrimitives = ec._triangleSet.size();
 
-    if (getSampleRatio()<1.0)
+
+    if (downSample)
     {
         while (!ec._edgeSet.empty() &&
                continueSimplification((*ec._edgeSet.begin())->getErrorMetric() , numOriginalPrimitives, ec._triangleSet.size()) &&
@@ -1800,8 +1802,7 @@ void Simplifier::simplify(osg::Geometry& geometry, const IndexList& protectedPoi
 
     if (_triStrip)
     {
-        osgUtil::TriStripVisitor stripper;
-        stripper.stripify(geometry);
+        osgUtil::optimizeMesh(&geometry);
     }
 
 }
