@@ -6,10 +6,20 @@
 #import <QuartzCore/QuartzCore.h>
 #import <UIKit/UIKit.h>
 
+void(^processInMainThread)(void(^funcBlock)()) = ^(void(^funcBlock)())
+{
+    if ([NSThread isMainThread])
+        funcBlock();
+    else
+        dispatch_sync(dispatch_get_main_queue(), ^{ funcBlock(); });
+};
+
 #if OSG_GLES1_FEATURES
     #import <OpenGLES/ES1/glext.h>
 #else
-    #import <OpenGLES/ES2/glext.h>
+    #if OSG_GLES2_FEATURES
+        #import <OpenGLES/ES2/glext.h>
+    #endif
     #if OSG_GLES3_FEATURES
         #import <OpenGLES/ES3/glext.h>
     #endif
@@ -38,8 +48,21 @@
     #define GL_STENCIL_ATTACHMENT_OES GL_STENCIL_ATTACHMENT
 
     #define GL_RGB5_A1_OES GL_RGB5_A1
+
+    #if OSG_GLES3_FEATURES && !OSG_GLES2_FEATURES
+        #define glRenderbufferStorageMultisampleAPPLE glRenderbufferStorageMultisample
+        #define glDiscardFramebufferEXT glInvalidateFramebuffer
+        #define glResolveMultisampleFramebufferAPPLE glResolveMultisampleFramebuffer
+
+        #define GL_DEPTH24_STENCIL8_OES GL_DEPTH24_STENCIL8
+        #define GL_DEPTH_COMPONENT24_OES GL_DEPTH_COMPONENT24
+        #define GL_READ_FRAMEBUFFER_APPLE GL_READ_FRAMEBUFFER
+        #define GL_DRAW_FRAMEBUFFER_APPLE GL_DRAW_FRAMEBUFFER
+    #endif
+
 #endif
 
+
 #include "IOSUtils.h"
 
 
@@ -296,7 +319,7 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 }
 
 //
-//Implement dealloc to destory our frame buffer
+//Implement dealloc to destroy our frame buffer
 //
 - (void) dealloc
 {
@@ -350,7 +373,14 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 
     glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
     glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
-    [_context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
+
+    if ([NSThread isMainThread])
+        [_context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
+    else
+        dispatch_sync(dispatch_get_main_queue(), ^{
+            [_context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];
+        });
+
     glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, _viewRenderbuffer);
 
     glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &_backingWidth);
@@ -433,14 +463,14 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
         glBindRenderbufferOES(GL_RENDERBUFFER_OES, _msaaRenderBuffer);
 
         // Samples is the amount of pixels the MSAA buffer uses to make one pixel on the render
-		// buffer. Use a small number like 2 for the 3G and below and 4 or more for newer models
-		// NOTE: Formats of draw and read buffers must be identical
+        // buffer. Use a small number like 2 for the 3G and below and 4 or more for newer models
+        // NOTE: Formats of draw and read buffers must be identical
 
-		GLenum internalFormat = GL_RGB5_A1_OES;
-#	if OSG_GLES3_FEATURES
-		if ([_context API] == kEAGLRenderingAPIOpenGLES3)
-			internalFormat = GL_RGBA8_OES;
-#	endif
+        GLenum internalFormat = GL_RGB5_A1_OES;
+#   if OSG_GLES3_FEATURES
+        if ([_context API] == kEAGLRenderingAPIOpenGLES3)
+            internalFormat = GL_RGBA8_OES;
+#   endif
 
         glRenderbufferStorageMultisampleAPPLE(GL_RENDERBUFFER_OES, _win->getTraits()->samples, internalFormat, _backingWidth, _backingHeight);
 
@@ -515,45 +545,47 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 
 
 #if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
-	if (_msaaFramebuffer) {
-		// Resolve the contents from the multisampling buffer into the resolve (view) buffer
+    if (_msaaFramebuffer) {
+        // Resolve the contents from the multisampling buffer into the resolve (view) buffer
         glBindFramebufferOES(GL_READ_FRAMEBUFFER_APPLE, _msaaFramebuffer);
         glBindFramebufferOES(GL_DRAW_FRAMEBUFFER_APPLE, _viewFramebuffer);
 
-#	if OSG_GLES3_FEATURES
+#   if OSG_GLES3_FEATURES
         if ([_context API] == kEAGLRenderingAPIOpenGLES3) {
-			glBlitFramebuffer(0, 0, _backingWidth, _backingHeight,
-							  0, 0, _backingWidth, _backingHeight,
-							  GL_COLOR_BUFFER_BIT, GL_LINEAR);
-		}
-		else
-			glResolveMultisampleFramebufferAPPLE();
-#	else
-		glResolveMultisampleFramebufferAPPLE();
-#	endif
+            glBlitFramebuffer(0, 0, _backingWidth, _backingHeight,
+                              0, 0, _backingWidth, _backingHeight,
+                              GL_COLOR_BUFFER_BIT, GL_LINEAR);
+        }
+#   if OSG_GLES2_FEATURES
+        else
+            glResolveMultisampleFramebufferAPPLE();
+#   endif
+#   else
+        glResolveMultisampleFramebufferAPPLE();
+#   endif
     }
 #endif
 
-	// Present Results step
+    // Present Results step
     glBindRenderbufferOES(GL_RENDERBUFFER_OES, _viewRenderbuffer);
     [_context presentRenderbuffer:GL_RENDERBUFFER_OES];
 
 #if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
-	if (_msaaFramebuffer) {
-		// Invalidate (discard) step (must be after present step)
-		GLenum attachments[] = {GL_DEPTH_ATTACHMENT_OES, GL_COLOR_ATTACHMENT0_OES};
-#	if OSG_GLES3_FEATURES
-		if ([_context API] == kEAGLRenderingAPIOpenGLES3)
-			glInvalidateFramebuffer(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
-		else
-			glDiscardFramebufferEXT(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
-#	else
-		glDiscardFramebufferEXT(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
-#	endif
-	}
+    if (_msaaFramebuffer) {
+        // Invalidate (discard) step (must be after present step)
+        GLenum attachments[] = {GL_DEPTH_ATTACHMENT_OES, GL_COLOR_ATTACHMENT0_OES};
+#   if OSG_GLES3_FEATURES
+        if ([_context API] == kEAGLRenderingAPIOpenGLES3)
+            glInvalidateFramebuffer(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
+        else
+            glDiscardFramebufferEXT(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
+#   else
+        glDiscardFramebufferEXT(GL_READ_FRAMEBUFFER_APPLE, 2, attachments);
+#   endif
+    }
 #endif
 
-	[self bindFrameBuffer];
+    [self bindFrameBuffer];
 }
 
 //
@@ -562,21 +594,21 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 - (void)bindFrameBuffer {
 #if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
     if (_msaaFramebuffer) {
-#	if OSG_GLES3_FEATURES
-		if ([_context API] == kEAGLRenderingAPIOpenGLES3) {
-			glBindFramebufferOES(GL_DRAW_FRAMEBUFFER_APPLE, _msaaFramebuffer);
-			glBindFramebufferOES(GL_READ_FRAMEBUFFER_APPLE, _viewFramebuffer);
-		}
-		else
-			glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);
-#	else
-		glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);
-#	endif
-	}
-	else
-		glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
+#   if OSG_GLES3_FEATURES
+        if ([_context API] == kEAGLRenderingAPIOpenGLES3) {
+            glBindFramebufferOES(GL_DRAW_FRAMEBUFFER_APPLE, _msaaFramebuffer);
+            glBindFramebufferOES(GL_READ_FRAMEBUFFER_APPLE, _viewFramebuffer);
+        }
+        else
+            glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);
+#   else
+        glBindFramebufferOES(GL_FRAMEBUFFER_OES, _msaaFramebuffer);
+#   endif
+    }
+    else
+        glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
 #else
-	glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
+    glBindFramebufferOES(GL_FRAMEBUFFER_OES, _viewFramebuffer);
 #endif
 }
 
@@ -698,8 +730,9 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 
 @interface GraphicsWindowIOSGLViewController : UIViewController
 {
-
 }
+
+- (void) viewDidAppear:(BOOL)animated;
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation;
 - (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation duration:(NSTimeInterval)duration;
 
@@ -707,6 +740,13 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 
 @implementation GraphicsWindowIOSGLViewController
 
+- (void) viewDidAppear:(BOOL)animated
+{
+    [super viewDidAppear:animated];
+    [UIViewController attemptRotationToDeviceOrientation];
+    if (self.view)
+        [self.view layoutSubviews];
+}
 
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
 {
@@ -761,6 +801,14 @@ typedef std::map<void*, unsigned int> TouchPointsIdMapping;
 using namespace osgIOS;
 namespace osgViewer {
 
+UIViewController* GraphicsWindowIOS::WindowData::getController() const
+{
+    return [_windowOrView isKindOfClass:[UIWindow class]]
+        ? ((UIWindow*)(_windowOrView)).rootViewController
+        : _parentController;
+}
+
+
 
 
 #pragma mark GraphicsWindowIOS
@@ -801,14 +849,17 @@ bool GraphicsWindowIOS::realizeImplementation()
 
     NSAutoreleasePool* pool = [[NSAutoreleasePool alloc] init];
 
-    BOOL bar_hidden = (_traits->windowDecoration) ? NO: YES;
-    #ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
+    processInMainThread(^
+    {
+        BOOL bar_hidden = (_traits->windowDecoration) ? NO: YES;
+#ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
     #if __IPHONE_OS_VERSION_MIN_REQUIRED > 30100
-        [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden withAnimation:UIStatusBarAnimationNone];
+            [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden withAnimation:UIStatusBarAnimationNone];
     #else
-        [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden animated:NO];
-    #endif
+            [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden animated:NO];
     #endif
+#endif
+    });
 
     //Get info about the requested screen
     IOSWindowingSystemInterface* wsi = dynamic_cast<IOSWindowingSystemInterface*>(osg::GraphicsContext::getWindowingSystemInterface());
@@ -824,7 +875,7 @@ bool GraphicsWindowIOS::realizeImplementation()
         wsi->getScreenSettings((*_traits), screenSettings);
         screen = wsi->getUIScreen((*_traits));
     }else{
-        OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create IOS windowing system, OSG will be unable to create a vaild gl context and will not be able to render." << std::endl;
+        OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create IOS windowing system, OSG will be unable to create a valid gl context and will not be able to render." << std::endl;
         return false;
     }
 
@@ -837,7 +888,7 @@ bool GraphicsWindowIOS::realizeImplementation()
         if (windowData->getWindowOrParentView())
         {
             _ownsWindow = false;
-            _window = windowData->getWindowOrParentView();
+            _window = (GraphicsWindowIOSWindow*)(windowData->getWindowOrParentView());
         }
 
         _deviceOrientationFlags = windowData->_deviceOrientationFlags;
@@ -849,7 +900,6 @@ bool GraphicsWindowIOS::realizeImplementation()
     if(_viewContentScaleFactor < 0.0f)
     {_viewContentScaleFactor = screenScaleFactor;}
 
-
     OSG_DEBUG << "GraphicsWindowIOS::realizeImplementation / ownsWindow: " << _ownsWindow << std::endl;
 
 
@@ -857,7 +907,7 @@ bool GraphicsWindowIOS::realizeImplementation()
     //but we need to create our views and windows in points. By default we create a full res buffer across all devices. This
     //means that for backward compatibility you need to set the windowData _viewContentScaleFactor to 1.0f and set the screen res to the
     //res of the older gen device.
-    CGRect window_bounds;
+    __block CGRect window_bounds;
     osg::Vec2 pointsOrigin = this->pixelToPoint(osg::Vec2(_traits->x, _traits->y));
     osg::Vec2 pointsSize = this->pixelToPoint(osg::Vec2(_traits->width, _traits->height));
 
@@ -870,8 +920,11 @@ bool GraphicsWindowIOS::realizeImplementation()
     //if we own the window we need to create one
     if (_ownsWindow)
     {
-        //create the IOS window object using the viewbounds (in points) required for our context size
-        _window = [[GraphicsWindowIOSWindow alloc] initWithFrame: window_bounds];// styleMask: style backing: NSBackingStoreBuffered defer: NO];
+        processInMainThread(^
+        {
+            //create the IOS window object using the viewbounds (in points) required for our context size
+            _window = [[GraphicsWindowIOSWindow alloc] initWithFrame: window_bounds];// styleMask: style backing: NSBackingStoreBuffered defer: NO];
+        });
 
         if (!_window) {
             OSG_WARN << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create GraphicsWindowIOSWindow can not display gl view" << std::endl;
@@ -880,7 +933,7 @@ bool GraphicsWindowIOS::realizeImplementation()
 
         OSG_DEBUG << "GraphicsWindowIOS::realizeImplementation: INFO: Created UIWindow with bounds '" << window_bounds.size.width << ", " << window_bounds.size.height << "' (points)." << std::endl;
 
-        //if the user has requested a differnet screenNum from default 0 get the UIScreen object and
+        //if the user has requested a different screenNum from default 0 get the UIScreen object and
         //apply to our window (this is for IPad external screens, I don't have one, so I've no idea if it works)
         //I'm also not sure if we should apply this to external windows also?
         if(_traits->screenNum > 0 && screen != nil)
@@ -895,42 +948,56 @@ bool GraphicsWindowIOS::realizeImplementation()
 #elif OSG_GLES2_FEATURES
     #if OSG_GLES3_FEATURES
         _context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES3];
-    #endif
-
-    if (!_context)
+        if (!_context)
+            _context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
+    #else
         _context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
+    #endif
+    
+#elif OSG_GLES3_FEATURES
+    _context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES3];
 #endif
 
     if (!_context || ![EAGLContext setCurrentContext:_context]) {
 
         #if OSG_GLES1_FEATURES
         OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create a valid OpenGLES1 context" << std::endl;
+        #elif OSG_GLES2_FEATURES && OSG_GLES3_FEATURES
+        OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create a valid OpenGLES3/OpenGLES2 context" << std::endl;
         #elif OSG_GLES2_FEATURES
-            #if OSG_GLES3_FEATURES
-        OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create a valid OpenGLES3 or OpenGLES2 context" << std::endl;
-            #else
         OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create a valid OpenGLES2 context" << std::endl;
-            #endif
+        #elif OSG_GLES3_FEATURES
+        OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create a valid OpenGLES3 context" << std::endl;
         #endif
+
         return false;
     }
 
-    //create the view to display our context in our window
-    CGRect gl_view_bounds = (_ownsWindow) ? [_window frame] : window_bounds;
-    GraphicsWindowIOSGLView* theView = [[ GraphicsWindowIOSGLView alloc ] initWithFrame: gl_view_bounds : this ];
+    __block GraphicsWindowIOSGLView* theView;
+    processInMainThread(^
+    {
+        //create the view to display our context in our window
+        CGRect gl_view_bounds = (_ownsWindow) ? [_window frame] : window_bounds;
+
+        theView = [[ GraphicsWindowIOSGLView alloc ] initWithFrame: gl_view_bounds : this ];
+        if (theView)
+        {
+            [theView setAutoresizingMask:  ( UIViewAutoresizingFlexibleWidth |  UIViewAutoresizingFlexibleHeight) ];
+
+            // Apply our content scale factor to our view, this is what converts the views points
+            // size to our desired context size.
+#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
+            theView.contentScaleFactor = _viewContentScaleFactor;
+#endif
+        }
+    });
+
     if(!theView)
     {
         OSG_FATAL << "GraphicsWindowIOS::realizeImplementation: ERROR: Failed to create GraphicsWindowIOSGLView, can not create frame buffers." << std::endl;
         return false;
     }
 
-    [theView setAutoresizingMask:  ( UIViewAutoresizingFlexibleWidth |  UIViewAutoresizingFlexibleHeight) ];
-
-    //Apply our content scale factor to our view, this is what converts the views points
-    //size to our desired context size.
-#if defined(__IPHONE_4_0) && (__IPHONE_OS_VERSION_MIN_REQUIRED >= __IPHONE_4_0)
-    theView.contentScaleFactor = _viewContentScaleFactor;
-#endif
     [theView setGraphicsWindow: this];
     [theView setOpenGLContext:_context];
     _view = theView;
@@ -939,23 +1006,33 @@ bool GraphicsWindowIOS::realizeImplementation()
 
     if (getDeviceOrientationFlags() != WindowData::IGNORE_ORIENTATION)
     {
-        _viewController = [[GraphicsWindowIOSGLViewController alloc] init];
-        _viewController.view = _view;
+        processInMainThread(^
+        {
+            _viewController = [[GraphicsWindowIOSGLViewController alloc] init];
+            _viewController.view = _view;
+        });
     }
 
-    // Attach view to window
-    [_window addSubview: _view];
-    if ([_window isKindOfClass:[UIWindow class]])
-        _window.rootViewController = _viewController;
+    processInMainThread(^
+    {
+        // Attach view to window
+        [_window addSubview: _view];
+
+        if ([_window isKindOfClass:[UIWindow class]]) // our controller is root
+            _window.rootViewController = _viewController;
+        else if (windowData->_parentController) // our controller is child
+            [windowData->_parentController addChildViewController:_viewController];
+
+        //if we own the window also make it visible
+        if (_ownsWindow)
+        {
+            //show window
+            [_window makeKeyAndVisible];
+        }
+    });
+
     [theView release];
 
-    //if we own the window also make it visible
-    if (_ownsWindow)
-    {
-
-        //show window
-        [_window makeKeyAndVisible];
-    }
 
     [pool release];
 
@@ -1013,13 +1090,8 @@ void GraphicsWindowIOS::closeImplementation()
 // makeCurrentImplementation
 // ----------------------------------------------------------------------------------------------------------
 
-bool GraphicsWindowIOS:: makeCurrentImplementation()
+void GraphicsWindowIOS::runOperations()
 {
-
-
-    //bind the context
-    [EAGLContext setCurrentContext:_context];
-
     if (_updateContext)
     {
         [_view destroyFramebuffer];
@@ -1027,6 +1099,15 @@ bool GraphicsWindowIOS:: makeCurrentImplementation()
 
         _updateContext = false;
     }
+
+    GraphicsContext::runOperations();
+}
+
+bool GraphicsWindowIOS::makeCurrentImplementation()
+{
+    //bind the context
+    [EAGLContext setCurrentContext:_context];
+
     //i think we also want to bind the frame buffer here
 //    [_view bindFrameBuffer];
 
@@ -1069,7 +1150,9 @@ bool GraphicsWindowIOS::setWindowDecorationImplementation(bool flag)
 {
     if (!_realized || !_ownsWindow) return false;
 
-    BOOL bar_hidden = (flag) ? NO: YES;
+    __block BOOL bar_hidden = (flag) ? NO: YES;
+    processInMainThread(^
+    {
     #ifdef __IPHONE_OS_VERSION_MIN_REQUIRED
     #if __IPHONE_OS_VERSION_MIN_REQUIRED > 30100
         [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden withAnimation:UIStatusBarAnimationNone];
@@ -1077,6 +1160,7 @@ bool GraphicsWindowIOS::setWindowDecorationImplementation(bool flag)
         [[UIApplication sharedApplication] setStatusBarHidden: bar_hidden animated:NO];
     #endif
     #endif
+    });
 
     return true;
 }
@@ -1087,8 +1171,11 @@ bool GraphicsWindowIOS::setWindowDecorationImplementation(bool flag)
 // ----------------------------------------------------------------------------------------------------------
 void GraphicsWindowIOS::grabFocus()
 {
-    //i think make key is the equivalent of focus on iphone
-    [_window makeKeyWindow];
+    processInMainThread(^
+    {
+        //i think make key is the equivalent of focus on iphone
+        [_window makeKeyWindow];
+    });
 }
 
 
@@ -1106,7 +1193,10 @@ void GraphicsWindowIOS::grabFocusIfPointerInWindow()
 // ----------------------------------------------------------------------------------------------------------
 void GraphicsWindowIOS::raiseWindow()
 {
-    [_window bringSubviewToFront:_view];
+    processInMainThread(^
+    {
+        [_window bringSubviewToFront:_view];
+    });
 }
 
 // ----------------------------------------------------------------------------------------------------------
@@ -1197,8 +1287,7 @@ osg::Vec2 GraphicsWindowIOS::pointToPixel(const osg::Vec2& point)
 
 osg::Vec2 GraphicsWindowIOS::pixelToPoint(const osg::Vec2& pixel)
 {
-    float scaler = 1.0f / _viewContentScaleFactor;
-    return pixel * scaler;
+    return pixel / _viewContentScaleFactor;
 }
 
 
@@ -1236,14 +1325,6 @@ public:
     }
 };
 
-}//end namspace
+REGISTER_WINDOWINGSYSTEMINTERFACE(IOS, ConcreteIOSWindowingSystemInterface)
 
-
-RegisterWindowingSystemInterfaceProxy<osgViewer::ConcreteIOSWindowingSystemInterface> createWindowingSystemInterfaceProxy;
-
-
-// declare C entry point for static compilation.
-extern "C" void graphicswindow_IOS(void)
-{
-    osg::GraphicsContext::setWindowingSystemInterface(new osgViewer::ConcreteIOSWindowingSystemInterface());
-}
+}//end namespace
