@@ -25,6 +25,7 @@
 
 #include <osgDB/DatabasePager>
 #include <osgDB/ImagePager>
+#include <osgDB/Registry>
 
 #include <osg/io_utils>
 
@@ -419,7 +420,7 @@ Renderer::Renderer(osg::Camera* camera):
     _sceneView[0]->setDefaults(sceneViewOptions);
     _sceneView[1]->setDefaults(sceneViewOptions);
 
-    if (ds->getUseSceneViewForStereoHint())
+    if (ds && ds->getUseSceneViewForStereoHint())
     {
         _sceneView[0]->setDisplaySettings(ds);
         _sceneView[1]->setDisplaySettings(ds);
@@ -536,17 +537,23 @@ void Renderer::updateSceneView(osgUtil::SceneView* sceneView)
 
     if (view && view->getFrameStamp())
     {
-        (*const_cast<osg::FrameStamp*>(sceneView->getFrameStamp())) = *(view->getFrameStamp());
+        (*sceneView->getFrameStamp()) = *(view->getFrameStamp());
     }
     else if (state->getFrameStamp())
     {
-        (*const_cast<osg::FrameStamp*>(sceneView->getFrameStamp())) = *(state->getFrameStamp());
+        (*sceneView->getFrameStamp()) = *(state->getFrameStamp());
+    }
+
+
+    if (view)
+    {
+        sceneView->setFusionDistance(view->getFusionDistanceMode(), view->getFusionDistanceValue());
     }
 
     osg::DisplaySettings* ds = _camera->getDisplaySettings() ?  _camera->getDisplaySettings() :
                                ((view &&view->getDisplaySettings()) ?  view->getDisplaySettings() :  osg::DisplaySettings::instance().get());
 
-    if (ds->getUseSceneViewForStereoHint())
+    if (ds && ds->getUseSceneViewForStereoHint())
     {
         sceneView->setDisplaySettings(ds);
     }
@@ -556,13 +563,21 @@ void Renderer::updateSceneView(osgUtil::SceneView* sceneView)
         _startTick = view->getStartTick();
         if (state) state->setStartTick(_startTick);
     }
+    else
+    {
+        osgViewer::GraphicsWindow* gw = dynamic_cast<osgViewer::GraphicsWindow*>(context);
+        if (gw)
+        {
+            _startTick = gw->getEventQueue()->getStartTick();
+            if (state) state->setStartTick(_startTick);
+        }
+    }
 }
 
 void Renderer::compile()
 {
     DEBUG_MESSAGE<<"Renderer::compile()"<<std::endl;
 
-
     _compileOnNextDraw = false;
 
     osgUtil::SceneView* sceneView = _sceneView[0].get();
@@ -574,7 +589,29 @@ void Renderer::compile()
     {
         osgUtil::GLObjectsVisitor glov;
         glov.setState(sceneView->getState());
-        sceneView->getSceneData()->accept(glov);
+
+        // collect stats if required
+        osg::View* view = _camera.valid() ? _camera->getView() : 0;
+        osg::Stats* stats = view ? view->getStats() : 0;
+        if (stats && stats->collectStats("compile"))
+        {
+            osg::ElapsedTime elapsedTime;
+
+            glov.compile(*(sceneView->getSceneData()));
+
+            double compileTime = elapsedTime.elapsedTime();
+
+            const osg::FrameStamp* fs = sceneView->getFrameStamp();
+            unsigned int frameNumber = fs ? fs->getFrameNumber() : 0;
+
+            stats->setAttribute(frameNumber, "compile", compileTime);
+
+            OSG_NOTICE<<"Compile time "<<compileTime*1000.0<<"ms"<<std::endl;
+        }
+        else
+        {
+            glov.compile(*(sceneView->getSceneData()));
+        }
     }
 
     sceneView->getState()->checkGLErrors("After Renderer::compile");
@@ -635,10 +672,6 @@ void Renderer::cull()
 
         // OSG_NOTICE<<"Culling buffer "<<_currentCull<<std::endl;
 
-        // pass on the fusion distance settings from the View to the SceneView
-        osgViewer::View* view = dynamic_cast<osgViewer::View*>(sceneView->getCamera()->getView());
-        if (view) sceneView->setFusionDistance(view->getFusionDistanceMode(), view->getFusionDistanceValue());
-
         osg::Stats* stats = sceneView->getCamera()->getStats();
         const osg::FrameStamp* fs = sceneView->getFrameStamp();
         unsigned int frameNumber = fs ? fs->getFrameNumber() : 0;
@@ -812,13 +845,8 @@ void Renderer::cull_draw()
         compile();
     }
 
-    osgViewer::View* view = dynamic_cast<osgViewer::View*>(_camera->getView());
-
     // OSG_NOTICE<<"RenderingOperation"<<std::endl;
 
-    // pass on the fusion distance settings from the View to the SceneView
-    if (view) sceneView->setFusionDistance(view->getFusionDistanceMode(), view->getFusionDistanceValue());
-
     osg::Stats* stats = sceneView->getCamera()->getStats();
     osg::State* state = sceneView->getState();
     const osg::FrameStamp* fs = sceneView->getFrameStamp();
@@ -909,7 +937,7 @@ void Renderer::operator () (osg::Object* object)
     osg::GraphicsContext* context = dynamic_cast<osg::GraphicsContext*>(object);
     if (context) operator()(context);
 
-    osg::Camera* camera = dynamic_cast<osg::Camera*>(object);
+    osg::Camera* camera =object->asCamera();
     if (camera) cull();
 }
 
@@ -925,6 +953,20 @@ void Renderer::operator () (osg::GraphicsContext* /*context*/)
     }
 }
 
+void Renderer::resizeGLObjectBuffers(unsigned int maxSize)
+{
+    if (_sceneView[0].valid()) _sceneView[0]->resizeGLObjectBuffers(maxSize);
+    if (_sceneView[1].valid()) _sceneView[1]->resizeGLObjectBuffers(maxSize);
+}
+
+void Renderer::releaseGLObjects(osg::State* state) const
+{
+    osgDB::Registry::instance()->releaseGLObjects(state);
+
+    if (_sceneView[0].valid()) _sceneView[0]->releaseGLObjects(state);
+    if (_sceneView[1].valid()) _sceneView[1]->releaseGLObjects(state);
+}
+
 void Renderer::release()
 {
     OSG_INFO<<"Renderer::release()"<<std::endl;
