@@ -161,9 +161,9 @@ class RTTCameraCullCallback : public osg::NodeCallback
             _tileData(tileData),
             _mt(mt) {}
 
-        virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
+        virtual void operator()(osg::Node* /*node*/, osg::NodeVisitor* nv)
         {
-            osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(nv);
+            osgUtil::CullVisitor* cv = nv->asCullVisitor();
 
             cv->pushProjectionMatrix(_tileData->projectionMatrix.get());
 
@@ -422,62 +422,43 @@ void MultipassTechnique::init()
     {
         osg::Geometry* geom = new osg::Geometry;
 
-        osg::Vec3Array* coords = new osg::Vec3Array(8);
-        (*coords)[0] = osg::Vec3d(0.0,0.0,0.0);
-        (*coords)[1] = osg::Vec3d(1.0,0.0,0.0);
-        (*coords)[2] = osg::Vec3d(1.0,1.0,0.0);
-        (*coords)[3] = osg::Vec3d(0.0,1.0,0.0);
-        (*coords)[4] = osg::Vec3d(0.0,0.0,1.0);
-        (*coords)[5] = osg::Vec3d(1.0,0.0,1.0);
-        (*coords)[6] = osg::Vec3d(1.0,1.0,1.0);
-        (*coords)[7] = osg::Vec3d(0.0,1.0,1.0);
-        geom->setVertexArray(coords);
-
         osg::Vec4Array* colours = new osg::Vec4Array(1);
         (*colours)[0].set(1.0f,1.0f,1.0,1.0f);
         geom->setColorArray(colours, osg::Array::BIND_OVERALL);
 
-        osg::DrawElementsUShort* drawElements = new osg::DrawElementsUShort(GL_QUADS);
-        // bottom
+        // triangle strip of cube based on : http://www.cs.umd.edu/gvil/papers/av_ts.pdf
+        osg::Vec3Array* coords = new osg::Vec3Array(8);
+        (*coords)[0] = osg::Vec3d(0.0,1.0,1.0); // Back-top-left
+        (*coords)[1] = osg::Vec3d(1.0,1.0,1.0); // Back-top-right
+        (*coords)[2] = osg::Vec3d(0.0,0.0,1.0); // Front-top-left
+        (*coords)[3] = osg::Vec3d(1.0,0.0,1.0); // Front-top-right
+        (*coords)[4] = osg::Vec3d(0.0,1.0,0.0); // Back-bottom-left
+        (*coords)[5] = osg::Vec3d(1.0,1.0,0.0); // Back-bottom-right
+        (*coords)[6] = osg::Vec3d(1.0,0.0,0.0); // Front-bottom-right
+        (*coords)[7] = osg::Vec3d(0.0,0.0,0.0); // Front-bottom-left
+        geom->setVertexArray(coords);
+
+        osg::DrawElementsUShort* drawElements = new osg::DrawElementsUShort(GL_TRIANGLE_STRIP);
+
         drawElements->push_back(3);
         drawElements->push_back(2);
-        drawElements->push_back(1);
-        drawElements->push_back(0);
-
-        // bottom
-        drawElements->push_back(7);//7623
         drawElements->push_back(6);
-        drawElements->push_back(2);
-        drawElements->push_back(3);
-
-        // left
-        drawElements->push_back(4);//4730
         drawElements->push_back(7);
-        drawElements->push_back(3);
-        drawElements->push_back(0);
-
-        // right
-        drawElements->push_back(1);//1265
-        drawElements->push_back(2);
-        drawElements->push_back(6);
-        drawElements->push_back(5);
-
-        // front
-        drawElements->push_back(5);//5401
         drawElements->push_back(4);
+        drawElements->push_back(2);
         drawElements->push_back(0);
-        drawElements->push_back(1);
 
-        // top
-        drawElements->push_back(4);//4567
-        drawElements->push_back(5);
+        drawElements->push_back(3);
+        drawElements->push_back(1);
         drawElements->push_back(6);
-        drawElements->push_back(7);
+        drawElements->push_back(5);
+        drawElements->push_back(4);
+        drawElements->push_back(1);
+        drawElements->push_back(0);
 
         geom->addPrimitiveSet(drawElements);
 
         geode->addDrawable(geom);
-
     }
 
     _transform = new osg::MatrixTransform;
@@ -928,33 +909,6 @@ void MultipassTechnique::backfaceSubgraphCullTraversal(osgUtil::CullVisitor* cv)
     cv->popStateSet();
 }
 
-class RTTBackfaceCameraCullCallback : public osg::NodeCallback
-{
-    public:
-
-        RTTBackfaceCameraCullCallback(MultipassTechnique::MultipassTileData* tileData, MultipassTechnique* mt):
-            _tileData(tileData),
-            _mt(mt) {}
-
-        virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
-        {
-            osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(nv);
-
-            cv->pushProjectionMatrix(_tileData->projectionMatrix.get());
-
-            _mt->backfaceSubgraphCullTraversal(cv);
-
-            cv->popProjectionMatrix();
-        }
-
-    protected:
-
-        virtual ~RTTBackfaceCameraCullCallback() {}
-
-        osg::observer_ptr<osgVolume::MultipassTechnique::MultipassTileData> _tileData;
-        osg::observer_ptr<osgVolume::MultipassTechnique> _mt;
-};
-
 class ShadingModelVisitor : public osgVolume::PropertyVisitor
 {
     public:
@@ -964,10 +918,10 @@ class ShadingModelVisitor : public osgVolume::PropertyVisitor
             _shadingModel(VolumeSettings::Standard),
             _usesTransferFunction(false) {}
 
-        virtual void apply(IsoSurfaceProperty& iso) { _shadingModel = VolumeSettings::Isosurface; }
-        virtual void apply(MaximumIntensityProjectionProperty& mip) { _shadingModel = VolumeSettings::MaximumIntensityProjection; }
-        virtual void apply(LightingProperty& lp) { _shadingModel = VolumeSettings::Light; }
-        virtual void apply(TransferFunctionProperty& tf) { _usesTransferFunction = true; }
+        virtual void apply(IsoSurfaceProperty& /*iso*/) { _shadingModel = VolumeSettings::Isosurface; }
+        virtual void apply(MaximumIntensityProjectionProperty& /*mip*/) { _shadingModel = VolumeSettings::MaximumIntensityProjection; }
+        virtual void apply(LightingProperty& /*lp*/) { _shadingModel = VolumeSettings::Light; }
+        virtual void apply(TransferFunctionProperty& /*tf*/) { _usesTransferFunction = true; }
         virtual void apply(VolumeSettings& vs) { _shadingModel = vs.getShadingModel(); }
 
         VolumeSettings::ShadingModel    _shadingModel;
@@ -1158,7 +1112,7 @@ void MultipassTechnique::traverse(osg::NodeVisitor& nv)
     {
         if (_volumeTile->getDirty()) _volumeTile->init();
 
-        osgUtil::UpdateVisitor* uv = dynamic_cast<osgUtil::UpdateVisitor*>(&nv);
+        osgUtil::UpdateVisitor* uv = nv.asUpdateVisitor();
         if (uv)
         {
             update(uv);
@@ -1168,7 +1122,7 @@ void MultipassTechnique::traverse(osg::NodeVisitor& nv)
     }
     else if (nv.getVisitorType()==osg::NodeVisitor::CULL_VISITOR)
     {
-        osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(&nv);
+        osgUtil::CullVisitor* cv = nv.asCullVisitor();
         if (cv)
         {
             cull(cv);
